import threading

from typing import Annotated

import typer
import uvicorn
import webview

from dbrownell_Common.Streams.DoneManager import DoneManager, Flags as DoneManagerFlags
from typer.core import TyperGroup

from dbrownell_BrythonWebviewTest.Impl import EntryPointUtils
from dbrownell_BrythonWebviewTest.web.Server import app as server


# ----------------------------------------------------------------------
class NaturalOrderGrouper(TyperGroup):  # noqa: D101
    # ----------------------------------------------------------------------
    def list_commands(self, *args, **kwargs) -> list[str]:  # noqa: ARG002, D102
        return list(self.commands.keys())  # pragma: no cover


# ----------------------------------------------------------------------
app = typer.Typer(
    cls=NaturalOrderGrouper,
    help=__doc__,
    no_args_is_help=True,
    pretty_exceptions_show_locals=False,
    pretty_exceptions_enable=False,
)


# ----------------------------------------------------------------------
@app.command("EntryPoint", no_args_is_help=False)
def EntryPoint(
    port: Annotated[
        int | None,
        typer.Option("--port", min=1024, max=65535, help="The port to run the server on."),
    ] = None,
    token: Annotated[
        str | None,
        typer.Option("--token", help="The token required to access protected endpoints."),
    ] = None,
    verbose: Annotated[  # noqa: FBT002
        bool,
        typer.Option("--verbose", help="Write verbose information to the terminal."),
    ] = False,
    debug: Annotated[  # noqa: FBT002
        bool,
        typer.Option("--debug", help="Write debug information to the terminal."),
    ] = False,
    debug_port: Annotated[
        int | None,
        typer.Option("--debug-port", min=1024, max=65535, help="The debug port used with the server."),
    ] = None,
    version: Annotated[  # noqa: ARG001
        bool | None,
        typer.Option(
            "--version",
            callback=EntryPointUtils.VersionCallback,
            is_eager=True,
            help="Display the version number and exit.",
        ),
    ] = None,
) -> None:
    """Run the app."""

    port = EntryPointUtils.ResolvePort(port)
    token = EntryPointUtils.ResolveToken(token)

    if debug_port is not None:
        if not debug:
            msg = "'--debug' must be specified when using '--debug-port'."
            raise typer.BadParameter(msg)

        webview.settings["REMOTE_DEBUGGING_PORT"] = debug_port

    with DoneManager.CreateCommandLine(
        flags=DoneManagerFlags.Create(verbose=verbose, debug=debug),
    ):
        server.state.token = token

        t = threading.Thread(
            target=uvicorn.run,
            args=(server,),
            daemon=True,
            kwargs={"host": "127.0.0.1", "port": port},
        )

        t.start()

        webview.create_window(
            "Brython Webview Test",
            f"http://127.0.0.1:{port}/",
        )

        webview.start(debug=debug)


# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
if __name__ == "__main__":
    app()  # pragma: no cover



import random
import secrets
import socket
import sys

import typer

from dbrownell_BrythonWebviewTest import __version__


# ----------------------------------------------------------------------
def GetUnusedPort() -> int:
    while True:
        port = random.randint(1024, 65535)  # noqa: S311
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        try:
            sock.bind(("127.0.0.1", port))
            sock.close()
            return port
        except OSError:
            pass


# ----------------------------------------------------------------------
def ResolvePort(port: int | None) -> int:
    return port or GetUnusedPort()


# ----------------------------------------------------------------------
def ResolveToken(token: str | None) -> str:
    return token or secrets.token_urlsafe(32)


# ----------------------------------------------------------------------
def VersionCallback(value: bool) -> None:  # noqa: FBT001
    if value:
        sys.stdout.write(__version__ + "\n")
        raise typer.Exit()



import sys
from pathlib import Path
from typing import Annotated

from fastapi import Depends, FastAPI, HTTPException, Request, status
from fastapi.responses import RedirectResponse
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates


# ----------------------------------------------------------------------
if getattr(sys, "frozen", False):
    # When running as a PyInstaller-bundled executable, bundled data files (templates, static assets)
    # are extracted to a temporary directory whose path is stored in sys._MEIPASS. The standard
    # __file__-based resolution doesn't work because the source .py files no longer exist on disk
    # in their original locations.
    _BASE_DIR = Path(sys._MEIPASS) / "dbrownell_BrythonWebviewTest" / "web"  # noqa: SLF001  # ty: ignore[unresolved-attribute]
else:
    _BASE_DIR = Path(__file__).parent

app = FastAPI()

# This value should be set externally
app.state.token: str | None = None

security = HTTPBearer()
templates = Jinja2Templates(directory=_BASE_DIR)


# ----------------------------------------------------------------------
def _ValidateToken(
    credentials: Annotated[HTTPAuthorizationCredentials, Depends(security)],
) -> str:
    assert app.state.token is not None

    token = credentials.credentials

    if token != app.state.token:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Invalid or missing token",
        )

    return token


# ----------------------------------------------------------------------
app.mount("/static", StaticFiles(directory=_BASE_DIR / "static", html=True))


# ----------------------------------------------------------------------
@app.get("/")
def index() -> object:
    return RedirectResponse("/static/index.html")


# ----------------------------------------------------------------------
@app.get("/index.py")
def index_py(request: Request) -> object:
    if app.state.token is None:
        raise HTTPException(
            status_code=status.HTTP_501_NOT_IMPLEMENTED,
            detail="Token validation has not been configured",
        )

    return templates.TemplateResponse(
        "index.jinja2.py",
        {
            "request": request,
            "token": app.state.token,
        },
    )


# ----------------------------------------------------------------------
@app.get("/add")
def add(num1: float, num2: float, _: Annotated[str, Depends(_ValidateToken)]) -> dict:
    return {"result": num1 + num2}


# ----------------------------------------------------------------------
@app.get("/sub")
def sub(num1: float, num2: float, _: Annotated[str, Depends(_ValidateToken)]) -> dict:
    return {"result": num1 - num2}


# ----------------------------------------------------------------------
@app.get("/mult")
def mult(num1: float, num2: float, _: Annotated[str, Depends(_ValidateToken)]) -> dict:
    return {"result": num1 * num2}


# ----------------------------------------------------------------------
@app.get("/div")
def div(num1: float, num2: float, _: Annotated[str, Depends(_ValidateToken)]) -> dict:
    result = "undefined" if num2 == 0.0 else num1 / num2

    return {"result": result}



from typing import Annotated

import typer
import uvicorn

from dbrownell_Common.Streams.DoneManager import DoneManager, Flags as DoneManagerFlags
from typer.core import TyperGroup

from dbrownell_BrythonWebviewTest.Impl import EntryPointUtils
from dbrownell_BrythonWebviewTest.web.Server import app as server


# ----------------------------------------------------------------------
class NaturalOrderGrouper(TyperGroup):  # noqa: D101
    # ----------------------------------------------------------------------
    def list_commands(self, *args, **kwargs) -> list[str]:  # noqa: ARG002, D102
        return list(self.commands.keys())  # pragma: no cover


# ----------------------------------------------------------------------
app = typer.Typer(
    cls=NaturalOrderGrouper,
    help=__doc__,
    no_args_is_help=True,
    pretty_exceptions_show_locals=False,
    pretty_exceptions_enable=False,
)


# ----------------------------------------------------------------------
@app.command("EntryPoint", no_args_is_help=False)
def EntryPoint(
    port: Annotated[
        int | None,
        typer.Option("--port", min=1024, max=65535, help="The port to run the server on."),
    ] = None,
    token: Annotated[
        str | None,
        typer.Option("--token", help="The token required to access protected endpoints."),
    ] = None,
    verbose: Annotated[  # noqa: FBT002
        bool,
        typer.Option("--verbose", help="Write verbose information to the terminal."),
    ] = False,
    debug: Annotated[  # noqa: FBT002
        bool,
        typer.Option("--debug", help="Write debug information to the terminal."),
    ] = False,
    version: Annotated[  # noqa: ARG001
        bool | None,
        typer.Option(
            "--version",
            callback=EntryPointUtils.VersionCallback,
            is_eager=True,
            help="Display the version number and exit.",
        ),
    ] = None,
) -> None:
    """Run the local server."""

    port = EntryPointUtils.ResolvePort(port)
    token = EntryPointUtils.ResolveToken(token)

    with DoneManager.CreateCommandLine(
        flags=DoneManagerFlags.Create(verbose=verbose, debug=debug),
    ):
        server.state.token = token

        uvicorn.run(
            server,
            host="127.0.0.1",
            port=port,
            reload=False,
            log_level="info",
        )


# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
# ----------------------------------------------------------------------
if __name__ == "__main__":
    app()  # pragma: no cover



from http import HTTPStatus

from browser import document, DOMEvent  # ty: ignore[unresolved-import]
from browser.aio import get, run  # ty: ignore[unresolved-import]


async def OnButtonClick(_: DOMEvent) -> None:
    num1 = document["num1"].value
    num2 = document["num2"].value

    result_div = document["result"]

    if not num1 or not num2:
        result_div.text = "Please enter valid numbers"
        return

    result = await get(
        f"/{document['operation'].value}",
        data={
            "num1": num1,
            "num2": num2,
        },
        headers={
            "Authorization": "Bearer {{ token }}",
        },
        cache=True,
    )

    if result.status != HTTPStatus.OK:  # noqa: SIM108
        result = f"Error: {result.status} - {result.data}"
    else:
        result = result.data

    result_div.text = result


document["submitBtn"].bind("click", lambda event: run(OnButtonClick(event)))
